/**
 * Copyright (C) 2010 Technische Universität Dresden
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * Dresden, University of Technology, Faculty of Computer Science
 * Computer Networks Group: http://www.rn.inf.tu-dresden.de
 * mobilis project: http://mobilisplatform.sourceforge.net
 */

package de.tudresden.inf.rn.mobilis.xmpp.beans;

import java.io.Serializable;

import org.xmlpull.v1.XmlPullParser;

/**
 * @author Benjamin Söllner, Robert Lübke
 */
public interface XMPPInfo extends Serializable {
	/**
	 * Parses the XML String of an XMPP stanza and saves all neccessary information. 
	 * @param parser XML Parser to use for parsing the XML String
	 * @throws Exception
	 */
	public void fromXML(XmlPullParser parser) throws Exception;
	/**
	 * Converts this XMPP stanza into its representation as XML string.
	 * @return XML string representation of this XMPP stanza 
	 */
	public String toXML(); 
	/**
	 * @return Child Element of this XMPP stanza
	 */
	public String getChildElement();
	/**
	 * @return Namespace of this XMPP stanza
	 */
	public String getNamespace();
}
